﻿using Plane.Copters;
using GalaSoft.MvvmLight;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Plane.FormationCreator.Formation
{
    public class CopterManager : ObservableObject
    {
        public CopterManager()
        {
            AppEx.Current.AppModeChanged += (sender, e) =>
            {
                RaisePropertyChanged(nameof(Copters));
            };
            App.Current.Exit += (sender, e) =>
            {
                Task.WhenAll(Copters.Select(c => c.DisconnectAsync()));
            };
        }


        public ObservableCollection<ICopter> Copters { get; } = new ObservableCollection<ICopter>();

        public ArrayList CopterStatus = new ArrayList();
        //public ObservableCollection<ICopter> Copters
        //{
        //    get
        //    {
        //        switch (AppEx.Instance.CurrentMode)
        //        {
        //            case AppMode.ControllingCopters:
        //            default:
        //                return CoptersForControlling;
        //            case AppMode.PreparedForRunningTasks:
        //            case AppMode.ModifyingTask:
        //                return CoptersForModifyingTask;
        //            case AppMode.RunningTasks:
        //                throw new NotImplementedException();
        //        }
        //    }
        //}


        public IEnumerable<ICopter> SelectedCopters { get { return _selectedCoptersGetter().Cast<ICopter>(); } }

        /// <summary>
        /// 注意：为避免多线程操作出问题，每次使用此属性时都会新建一个 List！
        /// </summary>
        public IEnumerable<ICopter> AcceptingControlCopters { get { return SelectedCopters.ToList(); } }

        private Func<IList> _selectedCoptersGetter;

        private Action<ICopter> _selectCopterAction;

        public void SetSelectionDelegates(Func<IList> selectedCoptersGetter, Action<ICopter> selectCopterAction)
        {
            _selectedCoptersGetter = selectedCoptersGetter;
            _selectCopterAction = selectCopterAction;
        }

        public event EventHandler<SelectedCoptersChangedEventArgs> SelectedCoptersChanged;

        public void RaiseSelectedCoptersChanged(IEnumerable<ICopter> addedCopters, IEnumerable<ICopter> removedCopters)
        {
            SelectedCoptersChanged?.Invoke(this, new SelectedCoptersChangedEventArgs { AddedCopters = addedCopters, RemovedCopters = removedCopters });
        }

        public void Select(ICopter copter)
        {
            _selectCopterAction(copter);
        }
    }

    public class SelectedCoptersChangedEventArgs : EventArgs
    {
        public IEnumerable<ICopter> AddedCopters { get; set; }
        public IEnumerable<ICopter> RemovedCopters { get; set; }
    }
}
